package com.redhat.training.claims;

import java.util.HashMap;

import com.redhat.training.claims.model.Claim;
import com.redhat.training.claims.model.ClaimClarificationRequests;
import com.redhat.training.claims.model.ClaimProcessingAborted;
import com.redhat.training.claims.model.ClaimResults;
import com.redhat.training.claims.model.Claimant;
import com.redhat.training.claims.model.Gender;

import org.kie.api.KieServices;
import org.kie.api.runtime.KieSession;
import org.kie.api.runtime.process.ProcessInstance;

/**
 * Intended to run an embedded process/rule engine with a KieClasspathContainer.
 * 
 * Uses a ClarificationTaskHandler to interact with the user about missing claim information.
 * 
 * You should probably look at claims-clients project for a KieScanner-based version of the same.
 */
public class SubmitClaimEmbedded {
    public static void main(String... args) {
        KieSession session = KieServices.Factory.get().getKieClasspathContainer().newKieSession();

        Claimant c = new Claimant();
        c.setName("Tom Jones");
        c.setGender(Gender.MALE);

        Claim cl = new Claim();
        cl.setId(234);
        cl.setAmount(500.00);
        cl.setReason("Test 123");
        cl.getInvolvedPersons().add(c);

        c.getClaims().add(cl);

        ClaimResults cr = new ClaimResults(cl.getId());
        ClaimClarificationRequests ccr = new ClaimClarificationRequests(cl.getId());
        ClaimProcessingAborted cpa = new ClaimProcessingAborted(cl.getId());

        session.getWorkItemManager().registerWorkItemHandler("Human Task", new ClarificationTaskHandler());

        HashMap<String, Object> params = new HashMap<>();
        params.put("claimant", c);
        params.put("claim", cl);
        params.put("claimResults", cr);
        params.put("claimClarificationRequests", ccr);
        params.put("claimProcessingAborted", cpa);
        ProcessInstance pi = session.startProcess("claims.ClaimRuleFlow", params);
        session.fireAllRules();

        String outcome;
        switch (pi.getState()) {
            case ProcessInstance.STATE_COMPLETED:
                outcome = "successfully processed";
                break;

            case ProcessInstance.STATE_ABORTED:
                outcome = "processed with errors";
                break;

            default:
                outcome = "in an unknown state";
        }

        System.out.println("Claim " + cl.getId() + " for claimant " + c.getName() + " " + outcome);
        if (pi.getState() == ProcessInstance.STATE_ABORTED) {
            for (String reason : cpa.getReasons()) {
                System.out.println(" - " + reason);
            }
        }
    }
}
